@@ -908,7 +908,7 @@ static int i802_sta_set_flags(void *priv, const u8 *addr,
 	if (total_flags & WLAN_STA_AUTHORIZED || !drv->ieee802_1x_active)
 		NLA_PUT_FLAG(flags, NL80211_STA_FLAG_AUTHORIZED);
 
-	if (total_flags & WLAN_STA_WME)
+	if (total_flags & WLAN_STA_WMM)
 		NLA_PUT_FLAG(flags, NL80211_STA_FLAG_WME);
 
 	if (total_flags & WLAN_STA_SHORT_PREAMBLE)
@@ -1432,7 +1432,7 @@ static int phy_info_handler(struct nl_msg *msg, void *arg)
 
 			/* crude heuristic */
 			if (mode->channels[idx].freq < 4000)
-				if (mode->channels[idx].freq == 2848)
+				if (mode->channels[idx].freq == 2484)
 					mode->channels[idx].chan = 14;
 				else
 					mode->channels[idx].chan = (mode->channels[idx].freq - 2407) / 5;
@@ -2380,7 +2380,7 @@ static void hostapd_wireless_event_rtm_newlink(struct i802_driver_data *drv,
 					       struct nlmsghdr *h, int len)
 {
 	struct ifinfomsg *ifi;
-	int attrlen, nlmsg_len, rta_len;
+	int attrlen, _nlmsg_len, rta_len;
 	struct rtattr *attr;
 
 	if (len < (int) sizeof(*ifi))
@@ -2391,13 +2391,13 @@ static void hostapd_wireless_event_rtm_newlink(struct i802_driver_data *drv,
 	/* TODO: use ifi->ifi_index to filter out wireless events from other
 	 * interfaces */
 
-	nlmsg_len = NLMSG_ALIGN(sizeof(struct ifinfomsg));
+	_nlmsg_len = NLMSG_ALIGN(sizeof(struct ifinfomsg));
 
-	attrlen = h->nlmsg_len - nlmsg_len;
+	attrlen = h->nlmsg_len - _nlmsg_len;
 	if (attrlen < 0)
 		return;
 
-	attr = (struct rtattr *) (((char *) ifi) + nlmsg_len);
+	attr = (struct rtattr *) (((char *) ifi) + _nlmsg_len);
 
 	rta_len = RTA_ALIGN(sizeof(struct rtattr));
 	while (RTA_OK(attr, attrlen)) {
